;################################################################################
;#										#
;# chipbasic - single chip basic computer with ATmega32				#
;# DataFlash memory interface							#
;# copyright (c) 2007 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
; send dataflash byte (tempreg1)
;------------------------------------------------------------------------------
dfl_out:	out	SPDR,tempreg1
dfl_out1:	sbis	SPSR,SPIF
		rjmp	dfl_out1
		in	tempreg1,SPDR
		ret
		
;------------------------------------------------------------------------------
; wait for Dataflash ready status
;------------------------------------------------------------------------------
dfl_stat:	cbi	PORTB,4			;start transfer
		ldi	tempreg1,0xd7		;command
		rcall	dfl_out			;output command
		clr	tempreg1		;dummy
		rcall	dfl_out
		sbi	PORTB,4			;stop transfer
		andi	tempreg1,0xbc
		cpi	tempreg1,0x24
		breq	dfl_stat
		clr 	ereg			;no error
		cpi	tempreg1,0xa4
		breq	dfl_stat1
		ldi	ereg,17			;dataflash error		
dfl_stat1:	ret	

;------------------------------------------------------------------------------
; read page X to buffer (tempreg2)
;------------------------------------------------------------------------------
dfl_pread:	rcall	dfl_stat		;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_pread1		;no
		cbi	PORTB,4
		ldi	tempreg1,0x53		;command
		sbrc	tempreg2,0
		ldi	tempreg1,0x55
		rcall	dfl_out			;output command
		lsl	XL
		rol	XH
		mov	tempreg1,XH		;High Page
		rcall	dfl_out
		mov	tempreg1,XL		;High Page
		rcall	dfl_out
		clr	tempreg1
		rcall	dfl_out
		sbi	PORTB,4			;stop transfer
dfl_pread1:	ret	

;------------------------------------------------------------------------------
; write buffer (tempreg2) to page X
;------------------------------------------------------------------------------
dfl_pwrite:	rcall	dfl_stat		;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_pwrite1		;no
		cbi	PORTB,4
		ldi	tempreg1,0x83		;command
		sbrc	tempreg2,0
		ldi	tempreg1,0x86
		rcall	dfl_out			;output command
		lsl	XL
		rol	XH
		mov	tempreg1,XH		;High Page
		rcall	dfl_out
		mov	tempreg1,XL		;High Page
		rcall	dfl_out
		clr	tempreg1
		rcall	dfl_out
		sbi	PORTB,4			;stop transfer
dfl_pwrite1:	ret	
		
;------------------------------------------------------------------------------
; read byte XL from buffer (tempreg3)
;------------------------------------------------------------------------------
dfl_br:		rcall	dfl_stat		;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_br1			;no
		rcall	dfl_rsr
		mov	XL,tempreg1
		clr	XH
		sbi	PORTB,4			;stop transfer
dfl_br1:	ret	

;------------------------------------------------------------------------------
; read word X from buffer (tempreg2)
;------------------------------------------------------------------------------
dfl_wr:		rcall	dfl_stat		;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_wr1			;no
		rcall	dfl_rsr
		mov	XL,tempreg1
		rcall	dfl_out
		mov	XH,tempreg1
		sbi	PORTB,4			;stop transfer
dfl_wr1:	ret	

;------------------------------------------------------------------------------
; write byte XL to buffer (tempreg2)
;------------------------------------------------------------------------------
dfl_bw:		rcall	dfl_stat		;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_bw1			;no
		rcall	dfl_wsr
		mov	tempreg1,XL
		rcall	dfl_out
		sbi	PORTB,4			;stop transfer
dfl_bw1:	ret	

;------------------------------------------------------------------------------
; write word X to buffer (tempreg2)
;------------------------------------------------------------------------------
dfl_ww:		rcall	dfl_stat			;wait for ready
		cpi	ereg,0			;OK
		brne	dfl_ww1			;no
		rcall	dfl_wsr
		mov	tempreg1,XL
		rcall	dfl_out
		mov	tempreg1,XH
		rcall	dfl_out
		sbi	PORTB,4			;stop transfer
dfl_ww1:	ret	
		
;------------------------------------------------------------------------------
; buffer write sub
;------------------------------------------------------------------------------
dfl_wsr:	cbi	PORTB,4
		ldi	tempreg1,0x84		;command
		sbrc	tempreg3,0
		ldi	tempreg1,0x87
		rcall	dfl_out			;output command
		clr	tempreg1		;page hi
		rcall	dfl_out
		clr	tempreg1		;page lo
		rcall	dfl_out
		lds 	tempreg1,bas_ram+10	;offset
		rjmp	dfl_out

;------------------------------------------------------------------------------
; buffer read sub
;------------------------------------------------------------------------------
dfl_rsr:	cbi	PORTB,4
		ldi	tempreg1,0xd4		;command
		sbrc	tempreg3,0
		ldi	tempreg1,0xd6
		rcall	dfl_out			;output command
		clr	tempreg1		;page hi
		rcall	dfl_out
		clr	tempreg1		;page lo
		rcall	dfl_out
		lds 	tempreg1,bas_ram+10	;offset
		rcall	dfl_out
		clr	tempreg1		;dummy1
		rcall	dfl_out
		rjmp	dfl_out
		